/*
 * C# / XNA  port of Bullet (c) 2011 Mark Neale <xexuxjy@hotmail.com>
 *
 * Bullet Continuous Collision Detection and Physics Library
 * Copyright (c) 2003-2008 Erwin Coumans  http://www.bulletphysics.com/
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from
 * the use of this software.
 * 
 * Permission is granted to anyone to use this software for any purpose, 
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 * 
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software
 *    in a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 * 3. This notice may not be removed or altered from any source distribution.
 */

using BulletMonogame.LinearMath;

namespace BulletMonogameDemos.Demos
{
    public static class DemoMeshes
    {

        public const int BUNNY_NUM_TRIANGLES = 902;
        public const int BUNNY_NUM_VERTICES = 453;
        public const int BUNNY_NUM_INDICES = BUNNY_NUM_TRIANGLES * 3;

        public const int TORUS_NUM_TRIANGLES = 600;
        public const int TORUS_NUM_VERTICES = 300;
        public const int TORUS_NUM_INDICES = TORUS_NUM_TRIANGLES * 3;
        
        public const int TaruVtxCount = 43;
        public const int TaruIdxCount = 132;

        //***************************THE FAMOUS BUNNY TRIMESH********************************************//

        // bit of a rubbish way of setting these up and not very efficient. should look at adding arrays 
        // as well as objectarray to mesh objects.
        public static ObjectArray<IndexedVector3> gBunnyVertices = new ObjectArray<IndexedVector3>(BUNNY_NUM_VERTICES);
        public static ObjectArray<IndexedVector3> gTorusVertices = new ObjectArray<IndexedVector3>(TORUS_NUM_VERTICES);
        public static ObjectArray<int> gBunnyIndices = new ObjectArray<int>(BUNNY_NUM_INDICES);
        public static ObjectArray<int> gTorusIndices = new ObjectArray<int>(TORUS_NUM_INDICES);

        static DemoMeshes()
        {
            gBunnyIndices.AddRange(gBunnyIndicesArray);
            gBunnyVertices.AddRange(gBunnyVerticesArary);
            gTorusIndices.AddRange(gTorusIndicesArray);
            gTorusVertices.AddRange(gTorusVerticesArray);
        }



        public static IndexedVector3[] gBunnyVerticesArary = new IndexedVector3[]
    {
new IndexedVector3(-0.334392f, 0.133007f, 0.062259f),
new IndexedVector3(-0.350189f, 0.150354f, -0.147769f),
new IndexedVector3(-0.234201f, 0.343811f, -0.174307f),
new IndexedVector3(-0.200259f, 0.285207f, 0.093749f),
new IndexedVector3(0.003520f, 0.475208f, -0.159365f),
new IndexedVector3(0.001856f, 0.419203f, 0.098582f),
new IndexedVector3(-0.252802f, 0.093666f, 0.237538f),
new IndexedVector3(-0.162901f, 0.237984f, 0.206905f),
new IndexedVector3(0.000865f, 0.318141f, 0.235370f),
new IndexedVector3(-0.414624f, 0.164083f, -0.278254f),
new IndexedVector3(-0.262213f, 0.357334f, -0.293246f),
new IndexedVector3(0.004628f, 0.482694f, -0.338626f),
new IndexedVector3(-0.402162f, 0.133528f, -0.443247f),
new IndexedVector3(-0.243781f, 0.324275f, -0.436763f),
new IndexedVector3(0.005293f, 0.437592f, -0.458332f),
new IndexedVector3(-0.339884f, -0.041150f, -0.668211f),
new IndexedVector3(-0.248382f, 0.255825f, -0.627493f),
new IndexedVector3(0.006261f, 0.376103f, -0.631506f),
new IndexedVector3(-0.216201f, -0.126776f, -0.886936f),
new IndexedVector3(-0.171075f, 0.011544f, -0.881386f),
new IndexedVector3(-0.181074f, 0.098223f, -0.814779f),
new IndexedVector3(-0.119891f, 0.218786f, -0.760153f),
new IndexedVector3(-0.078895f, 0.276780f, -0.739281f),
new IndexedVector3(0.006801f, 0.310959f, -0.735661f),
new IndexedVector3(-0.168842f, 0.102387f, -0.920381f),
new IndexedVector3(-0.104072f, 0.177278f, -0.952530f),
new IndexedVector3(-0.129704f, 0.211848f, -0.836678f),
new IndexedVector3(-0.099875f, 0.310931f, -0.799381f),
new IndexedVector3(0.007237f, 0.361687f, -0.794439f),
new IndexedVector3(-0.077913f, 0.258753f, -0.921640f),
new IndexedVector3(0.007957f, 0.282241f, -0.931680f),
new IndexedVector3(-0.252222f, -0.550401f, -0.557810f),
new IndexedVector3(-0.267633f, -0.603419f, -0.655209f),
new IndexedVector3(-0.446838f, -0.118517f, -0.466159f),
new IndexedVector3(-0.459488f, -0.093017f, -0.311341f),
new IndexedVector3(-0.370645f, -0.100108f, -0.159454f),
new IndexedVector3(-0.371984f, -0.091991f, -0.011044f),
new IndexedVector3(-0.328945f, -0.098269f, 0.088659f),
new IndexedVector3(-0.282452f, -0.018862f, 0.311501f),
new IndexedVector3(-0.352403f, -0.131341f, 0.144902f),
new IndexedVector3(-0.364126f, -0.200299f, 0.202388f),
new IndexedVector3(-0.283965f, -0.231869f, 0.023668f),
new IndexedVector3(-0.298943f, -0.155218f, 0.369716f),
new IndexedVector3(-0.293787f, -0.121856f, 0.419097f),
new IndexedVector3(-0.290163f, -0.290797f, 0.107824f),
new IndexedVector3(-0.264165f, -0.272849f, 0.036347f),
new IndexedVector3(-0.228567f, -0.372573f, 0.290309f),
new IndexedVector3(-0.190431f, -0.286997f, 0.421917f),
new IndexedVector3(-0.191039f, -0.240973f, 0.507118f),
new IndexedVector3(-0.287272f, -0.276431f, -0.065444f),
new IndexedVector3(-0.295675f, -0.280818f, -0.174200f),
new IndexedVector3(-0.399537f, -0.313131f, -0.376167f),
new IndexedVector3(-0.392666f, -0.488581f, -0.427494f),
new IndexedVector3(-0.331669f, -0.570185f, -0.466054f),
new IndexedVector3(-0.282290f, -0.618140f, -0.589220f),
new IndexedVector3(-0.374238f, -0.594882f, -0.323298f),
new IndexedVector3(-0.381071f, -0.629723f, -0.350777f),
new IndexedVector3(-0.382112f, -0.624060f, -0.221577f),
new IndexedVector3(-0.272701f, -0.566522f, 0.259157f),
new IndexedVector3(-0.256702f, -0.663406f, 0.286079f),
new IndexedVector3(-0.280948f, -0.428359f, 0.055790f),
new IndexedVector3(-0.184974f, -0.508894f, 0.326265f),
new IndexedVector3(-0.279971f, -0.526918f, 0.395319f),
new IndexedVector3(-0.282599f, -0.663393f, 0.412411f),
new IndexedVector3(-0.188329f, -0.475093f, 0.417954f),
new IndexedVector3(-0.263384f, -0.663396f, 0.466604f),
new IndexedVector3(-0.209063f, -0.663393f, 0.509344f),
new IndexedVector3(-0.002044f, -0.319624f, 0.553078f),
new IndexedVector3(-0.001266f, -0.371260f, 0.413296f),
new IndexedVector3(-0.219753f, -0.339762f, -0.040921f),
new IndexedVector3(-0.256986f, -0.282511f, -0.006349f),
new IndexedVector3(-0.271706f, -0.260881f, 0.001764f),
new IndexedVector3(-0.091191f, -0.419184f, -0.045912f),
new IndexedVector3(-0.114944f, -0.429752f, -0.124739f),
new IndexedVector3(-0.113970f, -0.382987f, -0.188540f),
new IndexedVector3(-0.243012f, -0.464942f, -0.242850f),
new IndexedVector3(-0.314815f, -0.505402f, -0.324768f),
new IndexedVector3(0.002774f, -0.437526f, -0.262766f),
new IndexedVector3(-0.072625f, -0.417748f, -0.221440f),
new IndexedVector3(-0.160112f, -0.476932f, -0.293450f),
new IndexedVector3(0.003859f, -0.453425f, -0.443916f),
new IndexedVector3(-0.120363f, -0.581567f, -0.438689f),
new IndexedVector3(-0.091499f, -0.584191f, -0.294511f),
new IndexedVector3(-0.116469f, -0.599861f, -0.188308f),
new IndexedVector3(-0.208032f, -0.513640f, -0.134649f),
new IndexedVector3(-0.235749f, -0.610017f, -0.040939f),
new IndexedVector3(-0.344916f, -0.622487f, -0.085380f),
new IndexedVector3(-0.336401f, -0.531864f, -0.212298f),
new IndexedVector3(0.001961f, -0.459550f, -0.135547f),
new IndexedVector3(-0.058296f, -0.430536f, -0.043440f),
new IndexedVector3(0.001378f, -0.449511f, -0.037762f),
new IndexedVector3(-0.130135f, -0.510222f, 0.079144f),
new IndexedVector3(0.000142f, -0.477549f, 0.157064f),
new IndexedVector3(-0.114284f, -0.453206f, 0.304397f),
new IndexedVector3(-0.000592f, -0.443558f, 0.285401f),
new IndexedVector3(-0.056215f, -0.663402f, 0.326073f),
new IndexedVector3(-0.026248f, -0.568010f, 0.273318f),
new IndexedVector3(-0.049261f, -0.531064f, 0.389854f),
new IndexedVector3(-0.127096f, -0.663398f, 0.479316f),
new IndexedVector3(-0.058384f, -0.663401f, 0.372891f),
new IndexedVector3(-0.303961f, 0.054199f, 0.625921f),
new IndexedVector3(-0.268594f, 0.193403f, 0.502766f),
new IndexedVector3(-0.277159f, 0.126123f, 0.443289f),
new IndexedVector3(-0.287605f, -0.005722f, 0.531844f),
new IndexedVector3(-0.231396f, -0.121289f, 0.587387f),
new IndexedVector3(-0.253475f, -0.081797f, 0.756541f),
new IndexedVector3(-0.195164f, -0.137969f, 0.728011f),
new IndexedVector3(-0.167673f, -0.156573f, 0.609388f),
new IndexedVector3(-0.145917f, -0.169029f, 0.697600f),
new IndexedVector3(-0.077776f, -0.214247f, 0.622586f),
new IndexedVector3(-0.076873f, -0.214971f, 0.696301f),
new IndexedVector3(-0.002341f, -0.233135f, 0.622859f),
new IndexedVector3(-0.002730f, -0.213526f, 0.691267f),
new IndexedVector3(-0.003136f, -0.192628f, 0.762731f),
new IndexedVector3(-0.056136f, -0.201222f, 0.763806f),
new IndexedVector3(-0.114589f, -0.166192f, 0.770723f),
new IndexedVector3(-0.155145f, -0.129632f, 0.791738f),
new IndexedVector3(-0.183611f, -0.058705f, 0.847012f),
new IndexedVector3(-0.165562f, 0.001980f, 0.833386f),
new IndexedVector3(-0.220084f, 0.019914f, 0.768935f),
new IndexedVector3(-0.255730f, 0.090306f, 0.670782f),
new IndexedVector3(-0.255594f, 0.113833f, 0.663389f),
new IndexedVector3(-0.226380f, 0.212655f, 0.617740f),
new IndexedVector3(-0.003367f, -0.195342f, 0.799680f),
new IndexedVector3(-0.029743f, -0.210508f, 0.827180f),
new IndexedVector3(-0.003818f, -0.194783f, 0.873636f),
new IndexedVector3(-0.004116f, -0.157907f, 0.931268f),
new IndexedVector3(-0.031280f, -0.184555f, 0.889476f),
new IndexedVector3(-0.059885f, -0.184448f, 0.841330f),
new IndexedVector3(-0.135333f, -0.164332f, 0.878200f),
new IndexedVector3(-0.085574f, -0.170948f, 0.925547f),
new IndexedVector3(-0.163833f, -0.094170f, 0.897114f),
new IndexedVector3(-0.138444f, -0.104250f, 0.945975f),
new IndexedVector3(-0.083497f, -0.084934f, 0.979607f),
new IndexedVector3(-0.004433f, -0.146642f, 0.985872f),
new IndexedVector3(-0.150715f, 0.032650f, 0.884111f),
new IndexedVector3(-0.135892f, -0.035520f, 0.945455f),
new IndexedVector3(-0.070612f, 0.036849f, 0.975733f),
new IndexedVector3(-0.004458f, -0.042526f, 1.015670f),
new IndexedVector3(-0.004249f, 0.046042f, 1.003240f),
new IndexedVector3(-0.086969f, 0.133224f, 0.947633f),
new IndexedVector3(-0.003873f, 0.161605f, 0.970499f),
new IndexedVector3(-0.125544f, 0.140012f, 0.917678f),
new IndexedVector3(-0.125651f, 0.250246f, 0.857602f),
new IndexedVector3(-0.003127f, 0.284070f, 0.878870f),
new IndexedVector3(-0.159174f, 0.125726f, 0.888878f),
new IndexedVector3(-0.183807f, 0.196970f, 0.844480f),
new IndexedVector3(-0.159890f, 0.291736f, 0.732480f),
new IndexedVector3(-0.199495f, 0.207230f, 0.779864f),
new IndexedVector3(-0.206182f, 0.164608f, 0.693257f),
new IndexedVector3(-0.186315f, 0.160689f, 0.817193f),
new IndexedVector3(-0.192827f, 0.166706f, 0.782271f),
new IndexedVector3(-0.175112f, 0.110008f, 0.860621f),
new IndexedVector3(-0.161022f, 0.057420f, 0.855111f),
new IndexedVector3(-0.172319f, 0.036155f, 0.816189f),
new IndexedVector3(-0.190318f, 0.064083f, 0.760605f),
new IndexedVector3(-0.195072f, 0.129179f, 0.731104f),
new IndexedVector3(-0.203126f, 0.410287f, 0.680536f),
new IndexedVector3(-0.216677f, 0.309274f, 0.642272f),
new IndexedVector3(-0.241515f, 0.311485f, 0.587832f),
new IndexedVector3(-0.002209f, 0.366663f, 0.749413f),
new IndexedVector3(-0.088230f, 0.396265f, 0.678635f),
new IndexedVector3(-0.170147f, 0.109517f, 0.840784f),
new IndexedVector3(-0.160521f, 0.067766f, 0.830650f),
new IndexedVector3(-0.181546f, 0.139805f, 0.812146f),
new IndexedVector3(-0.180495f, 0.148568f, 0.776087f),
new IndexedVector3(-0.180255f, 0.129125f, 0.744192f),
new IndexedVector3(-0.186298f, 0.078308f, 0.769352f),
new IndexedVector3(-0.167622f, 0.060539f, 0.806675f),
new IndexedVector3(-0.189876f, 0.102760f, 0.802582f),
new IndexedVector3(-0.108340f, 0.455446f, 0.657174f),
new IndexedVector3(-0.241585f, 0.527592f, 0.669296f),
new IndexedVector3(-0.265676f, 0.513366f, 0.634594f),
new IndexedVector3(-0.203073f, 0.478550f, 0.581526f),
new IndexedVector3(-0.266772f, 0.642330f, 0.602061f),
new IndexedVector3(-0.216961f, 0.564846f, 0.535435f),
new IndexedVector3(-0.202210f, 0.525495f, 0.475944f),
new IndexedVector3(-0.193888f, 0.467925f, 0.520606f),
new IndexedVector3(-0.265837f, 0.757267f, 0.500933f),
new IndexedVector3(-0.240306f, 0.653440f, 0.463215f),
new IndexedVector3(-0.309239f, 0.776868f, 0.304726f),
new IndexedVector3(-0.271009f, 0.683094f, 0.382018f),
new IndexedVector3(-0.312111f, 0.671099f, 0.286687f),
new IndexedVector3(-0.268791f, 0.624342f, 0.377231f),
new IndexedVector3(-0.302457f, 0.533996f, 0.360289f),
new IndexedVector3(-0.263656f, 0.529310f, 0.412564f),
new IndexedVector3(-0.282311f, 0.415167f, 0.447666f),
new IndexedVector3(-0.239201f, 0.442096f, 0.495604f),
new IndexedVector3(-0.220043f, 0.569026f, 0.445877f),
new IndexedVector3(-0.001263f, 0.395631f, 0.602029f),
new IndexedVector3(-0.057345f, 0.442535f, 0.572224f),
new IndexedVector3(-0.088927f, 0.506333f, 0.529106f),
new IndexedVector3(-0.125738f, 0.535076f, 0.612913f),
new IndexedVector3(-0.126251f, 0.577170f, 0.483159f),
new IndexedVector3(-0.149594f, 0.611520f, 0.557731f),
new IndexedVector3(-0.163188f, 0.660791f, 0.491080f),
new IndexedVector3(-0.172482f, 0.663387f, 0.415416f),
new IndexedVector3(-0.160464f, 0.591710f, 0.370659f),
new IndexedVector3(-0.156445f, 0.536396f, 0.378302f),
new IndexedVector3(-0.136496f, 0.444358f, 0.425226f),
new IndexedVector3(-0.095564f, 0.373768f, 0.473659f),
new IndexedVector3(-0.104146f, 0.315912f, 0.498104f),
new IndexedVector3(-0.000496f, 0.384194f, 0.473817f),
new IndexedVector3(-0.000183f, 0.297770f, 0.401486f),
new IndexedVector3(-0.129042f, 0.270145f, 0.434495f),
new IndexedVector3(0.000100f, 0.272963f, 0.349138f),
new IndexedVector3(-0.113060f, 0.236984f, 0.385554f),
new IndexedVector3(0.007260f, 0.016311f, -0.883396f),
new IndexedVector3(0.007865f, 0.122104f, -0.956137f),
new IndexedVector3(-0.032842f, 0.115282f, -0.953252f),
new IndexedVector3(-0.089115f, 0.108449f, -0.950317f),
new IndexedVector3(-0.047440f, 0.014729f, -0.882756f),
new IndexedVector3(-0.104458f, 0.013137f, -0.882070f),
new IndexedVector3(-0.086439f, -0.584866f, -0.608343f),
new IndexedVector3(-0.115026f, -0.662605f, -0.436732f),
new IndexedVector3(-0.071683f, -0.665372f, -0.606385f),
new IndexedVector3(-0.257884f, -0.665381f, -0.658052f),
new IndexedVector3(-0.272542f, -0.665381f, -0.592063f),
new IndexedVector3(-0.371322f, -0.665382f, -0.353620f),
new IndexedVector3(-0.372362f, -0.665381f, -0.224420f),
new IndexedVector3(-0.335166f, -0.665380f, -0.078623f),
new IndexedVector3(-0.225999f, -0.665375f, -0.038981f),
new IndexedVector3(-0.106719f, -0.665374f, -0.186351f),
new IndexedVector3(-0.081749f, -0.665372f, -0.292554f),
new IndexedVector3(0.006943f, -0.091505f, -0.858354f),
new IndexedVector3(0.006117f, -0.280985f, -0.769967f),
new IndexedVector3(0.004495f, -0.502360f, -0.559799f),
new IndexedVector3(-0.198638f, -0.302135f, -0.845816f),
new IndexedVector3(-0.237395f, -0.542544f, -0.587188f),
new IndexedVector3(-0.270001f, -0.279489f, -0.669861f),
new IndexedVector3(-0.134547f, -0.119852f, -0.959004f),
new IndexedVector3(-0.052088f, -0.122463f, -0.944549f),
new IndexedVector3(-0.124463f, -0.293508f, -0.899566f),
new IndexedVector3(-0.047616f, -0.289643f, -0.879292f),
new IndexedVector3(-0.168595f, -0.529132f, -0.654931f),
new IndexedVector3(-0.099793f, -0.515719f, -0.645873f),
new IndexedVector3(-0.186168f, -0.605282f, -0.724690f),
new IndexedVector3(-0.112970f, -0.583097f, -0.707469f),
new IndexedVector3(-0.108152f, -0.665375f, -0.700408f),
new IndexedVector3(-0.183019f, -0.665378f, -0.717630f),
new IndexedVector3(-0.349529f, -0.334459f, -0.511985f),
new IndexedVector3(-0.141182f, -0.437705f, -0.798194f),
new IndexedVector3(-0.212670f, -0.448725f, -0.737447f),
new IndexedVector3(-0.261111f, -0.414945f, -0.613835f),
new IndexedVector3(-0.077364f, -0.431480f, -0.778113f),
new IndexedVector3(0.005174f, -0.425277f, -0.651592f),
new IndexedVector3(0.089236f, -0.431732f, -0.777093f),
new IndexedVector3(0.271006f, -0.415749f, -0.610577f),
new IndexedVector3(0.223981f, -0.449384f, -0.734774f),
new IndexedVector3(0.153275f, -0.438150f, -0.796391f),
new IndexedVector3(0.358414f, -0.335529f, -0.507649f),
new IndexedVector3(0.193434f, -0.665946f, -0.715325f),
new IndexedVector3(0.118363f, -0.665717f, -0.699021f),
new IndexedVector3(0.123515f, -0.583454f, -0.706020f),
new IndexedVector3(0.196851f, -0.605860f, -0.722345f),
new IndexedVector3(0.109788f, -0.516035f, -0.644590f),
new IndexedVector3(0.178656f, -0.529656f, -0.652804f),
new IndexedVector3(0.061157f, -0.289807f, -0.878626f),
new IndexedVector3(0.138234f, -0.293905f, -0.897958f),
new IndexedVector3(0.066933f, -0.122643f, -0.943820f),
new IndexedVector3(0.149571f, -0.120281f, -0.957264f),
new IndexedVector3(0.280989f, -0.280321f, -0.666487f),
new IndexedVector3(0.246581f, -0.543275f, -0.584224f),
new IndexedVector3(0.211720f, -0.302754f, -0.843303f),
new IndexedVector3(0.086966f, -0.665627f, -0.291520f),
new IndexedVector3(0.110634f, -0.665702f, -0.185021f),
new IndexedVector3(0.228099f, -0.666061f, -0.036201f),
new IndexedVector3(0.337743f, -0.666396f, -0.074503f),
new IndexedVector3(0.376722f, -0.666513f, -0.219833f),
new IndexedVector3(0.377265f, -0.666513f, -0.349036f),
new IndexedVector3(0.281411f, -0.666217f, -0.588670f),
new IndexedVector3(0.267564f, -0.666174f, -0.654834f),
new IndexedVector3(0.080745f, -0.665602f, -0.605452f),
new IndexedVector3(0.122016f, -0.662963f, -0.435280f),
new IndexedVector3(0.095767f, -0.585141f, -0.607228f),
new IndexedVector3(0.118944f, 0.012799f, -0.880702f),
new IndexedVector3(0.061944f, 0.014564f, -0.882086f),
new IndexedVector3(0.104725f, 0.108156f, -0.949130f),
new IndexedVector3(0.048513f, 0.115159f, -0.952753f),
new IndexedVector3(0.112696f, 0.236643f, 0.386937f),
new IndexedVector3(0.128177f, 0.269757f, 0.436071f),
new IndexedVector3(0.102643f, 0.315600f, 0.499370f),
new IndexedVector3(0.094535f, 0.373481f, 0.474824f),
new IndexedVector3(0.136270f, 0.443946f, 0.426895f),
new IndexedVector3(0.157071f, 0.535923f, 0.380222f),
new IndexedVector3(0.161350f, 0.591224f, 0.372630f),
new IndexedVector3(0.173035f, 0.662865f, 0.417531f),
new IndexedVector3(0.162808f, 0.660299f, 0.493077f),
new IndexedVector3(0.148250f, 0.611070f, 0.559555f),
new IndexedVector3(0.125719f, 0.576790f, 0.484702f),
new IndexedVector3(0.123489f, 0.534699f, 0.614440f),
new IndexedVector3(0.087621f, 0.506066f, 0.530188f),
new IndexedVector3(0.055321f, 0.442365f, 0.572915f),
new IndexedVector3(0.219936f, 0.568361f, 0.448571f),
new IndexedVector3(0.238099f, 0.441375f, 0.498528f),
new IndexedVector3(0.281711f, 0.414315f, 0.451121f),
new IndexedVector3(0.263833f, 0.528513f, 0.415794f),
new IndexedVector3(0.303284f, 0.533081f, 0.363998f),
new IndexedVector3(0.269687f, 0.623528f, 0.380528f),
new IndexedVector3(0.314255f, 0.670153f, 0.290524f),
new IndexedVector3(0.272023f, 0.682273f, 0.385343f),
new IndexedVector3(0.311480f, 0.775931f, 0.308527f),
new IndexedVector3(0.240239f, 0.652714f, 0.466159f),
new IndexedVector3(0.265619f, 0.756464f, 0.504187f),
new IndexedVector3(0.192562f, 0.467341f, 0.522972f),
new IndexedVector3(0.201605f, 0.524885f, 0.478417f),
new IndexedVector3(0.215743f, 0.564193f, 0.538084f),
new IndexedVector3(0.264969f, 0.641527f, 0.605317f),
new IndexedVector3(0.201031f, 0.477940f, 0.584002f),
new IndexedVector3(0.263086f, 0.512567f, 0.637832f),
new IndexedVector3(0.238615f, 0.526867f, 0.672237f),
new IndexedVector3(0.105309f, 0.455123f, 0.658482f),
new IndexedVector3(0.183993f, 0.102195f, 0.804872f),
new IndexedVector3(0.161563f, 0.060042f, 0.808692f),
new IndexedVector3(0.180748f, 0.077754f, 0.771600f),
new IndexedVector3(0.175168f, 0.128588f, 0.746368f),
new IndexedVector3(0.175075f, 0.148030f, 0.778264f),
new IndexedVector3(0.175658f, 0.139265f, 0.814333f),
new IndexedVector3(0.154191f, 0.067291f, 0.832578f),
new IndexedVector3(0.163818f, 0.109013f, 0.842830f),
new IndexedVector3(0.084760f, 0.396004f, 0.679695f),
new IndexedVector3(0.238888f, 0.310760f, 0.590775f),
new IndexedVector3(0.213380f, 0.308625f, 0.644905f),
new IndexedVector3(0.199666f, 0.409678f, 0.683003f),
new IndexedVector3(0.190143f, 0.128597f, 0.733463f),
new IndexedVector3(0.184833f, 0.063516f, 0.762902f),
new IndexedVector3(0.166070f, 0.035644f, 0.818261f),
new IndexedVector3(0.154361f, 0.056943f, 0.857042f),
new IndexedVector3(0.168542f, 0.109489f, 0.862725f),
new IndexedVector3(0.187387f, 0.166131f, 0.784599f),
new IndexedVector3(0.180428f, 0.160135f, 0.819438f),
new IndexedVector3(0.201823f, 0.163991f, 0.695756f),
new IndexedVector3(0.194206f, 0.206635f, 0.782275f),
new IndexedVector3(0.155438f, 0.291260f, 0.734412f),
new IndexedVector3(0.177696f, 0.196424f, 0.846693f),
new IndexedVector3(0.152305f, 0.125256f, 0.890786f),
new IndexedVector3(0.119546f, 0.249876f, 0.859104f),
new IndexedVector3(0.118369f, 0.139643f, 0.919173f),
new IndexedVector3(0.079410f, 0.132973f, 0.948652f),
new IndexedVector3(0.062419f, 0.036648f, 0.976547f),
new IndexedVector3(0.127847f, -0.035919f, 0.947070f),
new IndexedVector3(0.143624f, 0.032206f, 0.885913f),
new IndexedVector3(0.074888f, -0.085173f, 0.980577f),
new IndexedVector3(0.130184f, -0.104656f, 0.947620f),
new IndexedVector3(0.156201f, -0.094653f, 0.899074f),
new IndexedVector3(0.077366f, -0.171194f, 0.926545f),
new IndexedVector3(0.127722f, -0.164729f, 0.879810f),
new IndexedVector3(0.052670f, -0.184618f, 0.842019f),
new IndexedVector3(0.023477f, -0.184638f, 0.889811f),
new IndexedVector3(0.022626f, -0.210587f, 0.827500f),
new IndexedVector3(0.223089f, 0.211976f, 0.620493f),
new IndexedVector3(0.251444f, 0.113067f, 0.666494f),
new IndexedVector3(0.251419f, 0.089540f, 0.673887f),
new IndexedVector3(0.214360f, 0.019258f, 0.771595f),
new IndexedVector3(0.158999f, 0.001490f, 0.835374f),
new IndexedVector3(0.176696f, -0.059249f, 0.849218f),
new IndexedVector3(0.148696f, -0.130091f, 0.793599f),
new IndexedVector3(0.108290f, -0.166528f, 0.772088f),
new IndexedVector3(0.049820f, -0.201382f, 0.764454f),
new IndexedVector3(0.071341f, -0.215195f, 0.697209f),
new IndexedVector3(0.073148f, -0.214475f, 0.623510f),
new IndexedVector3(0.140502f, -0.169461f, 0.699354f),
new IndexedVector3(0.163374f, -0.157073f, 0.611416f),
new IndexedVector3(0.189466f, -0.138550f, 0.730366f),
new IndexedVector3(0.247593f, -0.082554f, 0.759610f),
new IndexedVector3(0.227468f, -0.121982f, 0.590197f),
new IndexedVector3(0.284702f, -0.006586f, 0.535347f),
new IndexedVector3(0.275741f, 0.125287f, 0.446676f),
new IndexedVector3(0.266650f, 0.192594f, 0.506044f),
new IndexedVector3(0.300086f, 0.053287f, 0.629620f),
new IndexedVector3(0.055450f, -0.663935f, 0.375065f),
new IndexedVector3(0.122854f, -0.664138f, 0.482323f),
new IndexedVector3(0.046520f, -0.531571f, 0.391918f),
new IndexedVector3(0.024824f, -0.568450f, 0.275106f),
new IndexedVector3(0.053855f, -0.663931f, 0.328224f),
new IndexedVector3(0.112829f, -0.453549f, 0.305788f),
new IndexedVector3(0.131265f, -0.510617f, 0.080746f),
new IndexedVector3(0.061174f, -0.430716f, -0.042710f),
new IndexedVector3(0.341019f, -0.532887f, -0.208150f),
new IndexedVector3(0.347705f, -0.623533f, -0.081139f),
new IndexedVector3(0.238040f, -0.610732f, -0.038037f),
new IndexedVector3(0.211764f, -0.514274f, -0.132078f),
new IndexedVector3(0.120605f, -0.600219f, -0.186856f),
new IndexedVector3(0.096985f, -0.584476f, -0.293357f),
new IndexedVector3(0.127621f, -0.581941f, -0.437170f),
new IndexedVector3(0.165902f, -0.477425f, -0.291453f),
new IndexedVector3(0.077720f, -0.417975f, -0.220519f),
new IndexedVector3(0.320892f, -0.506363f, -0.320874f),
new IndexedVector3(0.248214f, -0.465684f, -0.239842f),
new IndexedVector3(0.118764f, -0.383338f, -0.187114f),
new IndexedVector3(0.118816f, -0.430106f, -0.123307f),
new IndexedVector3(0.094131f, -0.419464f, -0.044777f),
new IndexedVector3(0.274526f, -0.261706f, 0.005110f),
new IndexedVector3(0.259842f, -0.283292f, -0.003185f),
new IndexedVector3(0.222861f, -0.340431f, -0.038210f),
new IndexedVector3(0.204445f, -0.664380f, 0.513353f),
new IndexedVector3(0.259286f, -0.664547f, 0.471281f),
new IndexedVector3(0.185402f, -0.476020f, 0.421718f),
new IndexedVector3(0.279163f, -0.664604f, 0.417328f),
new IndexedVector3(0.277157f, -0.528122f, 0.400208f),
new IndexedVector3(0.183069f, -0.509812f, 0.329995f),
new IndexedVector3(0.282599f, -0.429210f, 0.059242f),
new IndexedVector3(0.254816f, -0.664541f, 0.290687f),
new IndexedVector3(0.271436f, -0.567707f, 0.263966f),
new IndexedVector3(0.386561f, -0.625221f, -0.216870f),
new IndexedVector3(0.387086f, -0.630883f, -0.346073f),
new IndexedVector3(0.380021f, -0.596021f, -0.318679f),
new IndexedVector3(0.291269f, -0.619007f, -0.585707f),
new IndexedVector3(0.339280f, -0.571198f, -0.461946f),
new IndexedVector3(0.400045f, -0.489778f, -0.422640f),
new IndexedVector3(0.406817f, -0.314349f, -0.371230f),
new IndexedVector3(0.300588f, -0.281718f, -0.170549f),
new IndexedVector3(0.290866f, -0.277304f, -0.061905f),
new IndexedVector3(0.187735f, -0.241545f, 0.509437f),
new IndexedVector3(0.188032f, -0.287569f, 0.424234f),
new IndexedVector3(0.227520f, -0.373262f, 0.293102f),
new IndexedVector3(0.266526f, -0.273650f, 0.039597f),
new IndexedVector3(0.291592f, -0.291676f, 0.111386f),
new IndexedVector3(0.291914f, -0.122741f, 0.422683f),
new IndexedVector3(0.297574f, -0.156119f, 0.373368f),
new IndexedVector3(0.286603f, -0.232731f, 0.027162f),
new IndexedVector3(0.364663f, -0.201399f, 0.206850f),
new IndexedVector3(0.353855f, -0.132408f, 0.149228f),
new IndexedVector3(0.282208f, -0.019715f, 0.314960f),
new IndexedVector3(0.331187f, -0.099266f, 0.092701f),
new IndexedVector3(0.375463f, -0.093120f, -0.006467f),
new IndexedVector3(0.375917f, -0.101236f, -0.154882f),
new IndexedVector3(0.466635f, -0.094416f, -0.305669f),
new IndexedVector3(0.455805f, -0.119881f, -0.460632f),
new IndexedVector3(0.277465f, -0.604242f, -0.651871f),
new IndexedVector3(0.261022f, -0.551176f, -0.554667f),
new IndexedVector3(0.093627f, 0.258494f, -0.920589f),
new IndexedVector3(0.114248f, 0.310608f, -0.798070f),
new IndexedVector3(0.144232f, 0.211434f, -0.835001f),
new IndexedVector3(0.119916f, 0.176940f, -0.951159f),
new IndexedVector3(0.184061f, 0.101854f, -0.918220f),
new IndexedVector3(0.092431f, 0.276521f, -0.738231f),
new IndexedVector3(0.133504f, 0.218403f, -0.758602f),
new IndexedVector3(0.194987f, 0.097655f, -0.812476f),
new IndexedVector3(0.185542f, 0.011005f, -0.879202f),
new IndexedVector3(0.230315f, -0.127450f, -0.884202f),
new IndexedVector3(0.260471f, 0.255056f, -0.624378f),
new IndexedVector3(0.351567f, -0.042194f, -0.663976f),
new IndexedVector3(0.253742f, 0.323524f, -0.433716f),
new IndexedVector3(0.411612f, 0.132299f, -0.438264f),
new IndexedVector3(0.270513f, 0.356530f, -0.289984f),
new IndexedVector3(0.422146f, 0.162819f, -0.273130f),
new IndexedVector3(0.164724f, 0.237490f, 0.208912f),
new IndexedVector3(0.253806f, 0.092900f, 0.240640f),
new IndexedVector3(0.203608f, 0.284597f, 0.096223f),
new IndexedVector3(0.241006f, 0.343093f, -0.171396f),
new IndexedVector3(0.356076f, 0.149288f, -0.143443f),
new IndexedVector3(0.337656f, 0.131992f, 0.066374f)
};

        static int[] gBunnyIndicesArray = new int[]  {
	126,134,133,
	342,138,134,
	133,134,138,
	126,342,134,
	312,316,317,
	169,163,162,
	312,317,319,
	312,319,318,
	169,162,164,
	169,168,163,
	312,314,315,
	169,164,165,
	169,167,168,
	312,315,316,
	312,313,314,
	169,165,166,
	169,166,167,
	312,318,313,
	308,304,305,
	308,305,306,
	179,181,188,
	177,173,175,
	177,175,176,
	302,293,300,
	322,294,304,
	188,176,175,
	188,175,179,
	158,177,187,
	305,293,302,
	305,302,306,
	322,304,308,
	188,181,183,
	158,173,177,
	293,298,300,
	304,294,296,
	304,296,305,
	185,176,188,
	185,188,183,
	187,177,176,
	187,176,185,
	305,296,298,
	305,298,293,
	436,432, 28,
	436, 28, 23,
	434,278,431,
	 30,208,209,
	 30,209, 29,
	 19, 20, 24,
	208,207,211,
	208,211,209,
	 19,210,212,
	433,434,431,
	433,431,432,
	433,432,436,
	436,437,433,
	277,275,276,
	277,276,278,
	209,210, 25,
	 21, 26, 24,
	 21, 24, 20,
	 25, 26, 27,
	 25, 27, 29,
	435,439,277,
	439,275,277,
	432,431, 30,
	432, 30, 28,
	433,437,438,
	433,438,435,
	434,277,278,
	 24, 25,210,
	 24, 26, 25,
	 29, 27, 28,
	 29, 28, 30,
	 19, 24,210,
	208, 30,431,
	208,431,278,
	435,434,433,
	435,277,434,
	 25, 29,209,
	 27, 22, 23,
	 27, 23, 28,
	 26, 22, 27,
	 26, 21, 22,
	212,210,209,
	212,209,211,
	207,208,278,
	207,278,276,
	439,435,438,
	 12,  9, 10,
	 12, 10, 13,
	  2,  3,  5,
	  2,  5,  4,
	 16, 13, 14,
	 16, 14, 17,
	 22, 21, 16,
	 13, 10, 11,
	 13, 11, 14,
	  1,  0,  3,
	  1,  3,  2,
	 15, 12, 16,
	 19, 18, 15,
	 19, 15, 16,
	 19, 16, 20,
	  9,  1,  2,
	  9,  2, 10,
	  3,  7,  8,
	  3,  8,  5,
	 16, 17, 23,
	 16, 23, 22,
	 21, 20, 16,
	 10,  2,  4,
	 10,  4, 11,
	  0,  6,  7,
	  0,  7,  3,
	 12, 13, 16,
	451,446,445,
	451,445,450,
	442,440,439,
	442,439,438,
	442,438,441,
	421,420,422,
	412,411,426,
	412,426,425,
	408,405,407,
	413, 67, 68,
	413, 68,414,
	391,390,412,
	 80,384,386,
	404,406,378,
	390,391,377,
	390,377, 88,
	400,415,375,
	398,396,395,
	398,395,371,
	398,371,370,
	112,359,358,
	112,358,113,
	351,352,369,
	125,349,348,
	345,343,342,
	342,340,339,
	341,335,337,
	328,341,327,
	331,323,333,
	331,322,323,
	327,318,319,
	327,319,328,
	315,314,324,
	302,300,301,
	302,301,303,
	320,311,292,
	285,284,289,
	310,307,288,
	310,288,290,
	321,350,281,
	321,281,282,
	423,448,367,
	272,273,384,
	272,384,274,
	264,265,382,
	264,382,383,
	440,442,261,
	440,261,263,
	252,253,254,
	252,254,251,
	262,256,249,
	262,249,248,
	228,243,242,
	228, 31,243,
	213,215,238,
	213,238,237,
	 19,212,230,
	224,225,233,
	224,233,231,
	217,218, 56,
	217, 56, 54,
	217,216,239,
	217,239,238,
	217,238,215,
	218,217,215,
	218,215,214,
	  6,102,206,
	186,199,200,
	197,182,180,
	170,171,157,
	201,200,189,
	170,190,191,
	170,191,192,
	175,174,178,
	175,178,179,
	168,167,155,
	122,149,158,
	122,158,159,
	135,153,154,
	135,154,118,
	143,140,141,
	143,141,144,
	132,133,136,
	130,126,133,
	124,125,127,
	122,101,100,
	122,100,121,
	110,108,107,
	110,107,109,
	 98, 99, 97,
	 98, 97, 64,
	 98, 64, 66,
	 87, 55, 57,
	 83, 82, 79,
	 83, 79, 84,
	 78, 74, 50,
	 49, 71, 41,
	 49, 41, 37,
	 49, 37, 36,
	 58, 44, 60,
	 60, 59, 58,
	 51, 34, 33,
	 39, 40, 42,
	 39, 42, 38,
	243,240, 33,
	243, 33,229,
	 39, 38,  6,
	 44, 46, 40,
	 55, 56, 57,
	 64, 62, 65,
	 64, 65, 66,
	 41, 71, 45,
	 75, 50, 51,
	 81, 79, 82,
	 77, 88, 73,
	 93, 92, 94,
	 68, 47, 46,
	 96, 97, 99,
	 96, 99, 95,
	110,109,111,
	111,112,110,
	114,113,123,
	114,123,124,
	132,131,129,
	133,137,136,
	135,142,145,
	145,152,135,
	149,147,157,
	157,158,149,
	164,150,151,
	153,163,168,
	153,168,154,
	185,183,182,
	185,182,184,
	161,189,190,
	200,199,191,
	200,191,190,
	180,178,195,
	180,195,196,
	102,101,204,
	102,204,206,
	 43, 48,104,
	 43,104,103,
	216,217, 54,
	216, 54, 32,
	207,224,231,
	230,212,211,
	230,211,231,
	227,232,241,
	227,241,242,
	235,234,241,
	235,241,244,
	430,248,247,
	272,274,253,
	272,253,252,
	439,260,275,
	225,224,259,
	225,259,257,
	269,270,407,
	269,407,405,
	270,269,273,
	270,273,272,
	273,269,268,
	273,268,267,
	273,267,266,
	273,266,265,
	273,265,264,
	448,279,367,
	281,350,368,
	285,286,301,
	290,323,310,
	290,311,323,
	282,281,189,
	292,311,290,
	292,290,291,
	307,306,302,
	307,302,303,
	316,315,324,
	316,324,329,
	331,351,350,
	330,334,335,
	330,335,328,
	341,337,338,
	344,355,354,
	346,345,348,
	346,348,347,
	364,369,352,
	364,352,353,
	365,363,361,
	365,361,362,
	376,401,402,
	373,372,397,
	373,397,400,
	376, 92,377,
	381,378,387,
	381,387,385,
	386, 77, 80,
	390,389,412,
	416,417,401,
	403,417,415,
	408,429,430,
	419,423,418,
	427,428,444,
	427,444,446,
	437,436,441,
	450,445, 11,
	450, 11,  4,
	447,449,  5,
	447,  5,  8,
	441,438,437,
	425,426,451,
	425,451,452,
	417,421,415,
	408,407,429,
	399,403,400,
	399,400,397,
	394,393,416,
	389,411,412,
	386,383,385,
	408,387,378,
	408,378,406,
	377,391,376,
	 94,375,415,
	372,373,374,
	372,374,370,
	359,111,360,
	359,112,111,
	113,358,349,
	113,349,123,
	346,343,345,
	343,340,342,
	338,336,144,
	338,144,141,
	327,341,354,
	327,354,326,
	331,350,321,
	331,321,322,
	314,313,326,
	314,326,325,
	300,298,299,
	300,299,301,
	288,287,289,
	189,292,282,
	287,288,303,
	284,285,297,
	368,280,281,
	448,447,279,
	274,226,255,
	267,268,404,
	267,404,379,
	429,262,430,
	439,440,260,
	257,258,249,
	257,249,246,
	430,262,248,
	234,228,242,
	234,242,241,
	237,238,239,
	237,239,236,
	 15, 18,227,
	 15,227,229,
	222,223, 82,
	222, 82, 83,
	214,215,213,
	214,213, 81,
	 38,102,  6,
	122,159,200,
	122,200,201,
	174,171,192,
	174,192,194,
	197,193,198,
	190,170,161,
	181,179,178,
	181,178,180,
	166,156,155,
	163,153,152,
	163,152,162,
	120,156,149,
	120,149,121,
	152,153,135,
	140,143,142,
	135,131,132,
	135,132,136,
	130,129,128,
	130,128,127,
	100,105,119,
	100,119,120,
	106,104,107,
	106,107,108,
	 91, 95, 59,
	 93, 94, 68,
	 91, 89, 92,
	 76, 53, 55,
	 76, 55, 87,
	 81, 78, 79,
	 74, 73, 49,
	 69, 60, 45,
	 58, 62, 64,
	 58, 64, 61,
	 53, 31, 32,
	 32, 54, 53,
	 42, 43, 38,
	 35, 36,  0,
	 35,  0,  1,
	 34, 35,  1,
	 34,  1,  9,
	 44, 40, 41,
	 44, 41, 45,
	 33,240, 51,
	 63, 62, 58,
	 63, 58, 59,
	 45, 71, 70,
	 76, 75, 51,
	 76, 51, 52,
	 86, 85, 84,
	 86, 84, 87,
	 89, 72, 73,
	 89, 73, 88,
	 91, 92, 96,
	 91, 96, 95,
	 72, 91, 60,
	 72, 60, 69,
	104,106,105,
	119,105,117,
	119,117,118,
	124,127,128,
	117,116,129,
	117,129,131,
	118,117,131,
	135,140,142,
	146,150,152,
	146,152,145,
	149,122,121,
	166,165,151,
	166,151,156,
	158,172,173,
	161,160,189,
	199,198,193,
	199,193,191,
	204,201,202,
	178,174,194,
	200,159,186,
	109, 48, 67,
	 48,107,104,
	216, 32,236,
	216,236,239,
	223,214, 81,
	223, 81, 82,
	 33, 12, 15,
	 32,228,234,
	 32,234,236,
	240, 31, 52,
	256,255,246,
	256,246,249,
	258,263,248,
	258,248,249,
	275,260,259,
	275,259,276,
	207,276,259,
	270,271,429,
	270,429,407,
	413,418,366,
	413,366,365,
	368,367,279,
	368,279,280,
	303,301,286,
	303,286,287,
	283,282,292,
	283,292,291,
	320,292,189,
	298,296,297,
	298,297,299,
	318,327,326,
	318,326,313,
	329,330,317,
	336,333,320,
	326,354,353,
	334,332,333,
	334,333,336,
	342,339,139,
	342,139,138,
	345,342,126,
	347,357,356,
	369,368,351,
	363,356,357,
	363,357,361,
	366,367,368,
	366,368,369,
	375,373,400,
	 92, 90,377,
	409,387,408,
	386,385,387,
	386,387,388,
	412,394,391,
	396,398,399,
	408,406,405,
	415,421,419,
	415,419,414,
	425,452,448,
	425,448,424,
	444,441,443,
	448,452,449,
	448,449,447,
	446,444,443,
	446,443,445,
	250,247,261,
	250,261,428,
	421,422,423,
	421,423,419,
	427,410,250,
	417,403,401,
	403,402,401,
	420,392,412,
	420,412,425,
	420,425,424,
	386,411,389,
	383,382,381,
	383,381,385,
	378,379,404,
	372,371,395,
	372,395,397,
	371,372,370,
	361,359,360,
	361,360,362,
	368,350,351,
	349,347,348,
	356,355,344,
	356,344,346,
	344,341,340,
	344,340,343,
	338,337,336,
	328,335,341,
	324,352,351,
	324,351,331,
	320,144,336,
	314,325,324,
	322,308,309,
	310,309,307,
	287,286,289,
	203,280,279,
	203,279,205,
	297,295,283,
	297,283,284,
	447,205,279,
	274,384, 80,
	274, 80,226,
	266,267,379,
	266,379,380,
	225,257,246,
	225,246,245,
	256,254,253,
	256,253,255,
	430,247,250,
	226,235,244,
	226,244,245,
	232,233,244,
	232,244,241,
	230, 18, 19,
	 32, 31,228,
	219,220, 86,
	219, 86, 57,
	226,213,235,
	206,  7,  6,
	122,201,101,
	201,204,101,
	180,196,197,
	170,192,171,
	200,190,189,
	194,193,195,
	183,181,180,
	183,180,182,
	155,154,168,
	149,156,151,
	149,151,148,
	155,156,120,
	145,142,143,
	145,143,146,
	136,137,140,
	133,132,130,
	128,129,116,
	100,120,121,
	110,112,113,
	110,113,114,
	 66, 65, 63,
	 66, 63, 99,
	 66, 99, 98,
	 96, 46, 61,
	 89, 88, 90,
	 86, 87, 57,
	 80, 78, 81,
	 72, 69, 49,
	 67, 48, 47,
	 67, 47, 68,
	 56, 55, 53,
	 50, 49, 36,
	 50, 36, 35,
	 40, 39, 41,
	242,243,229,
	242,229,227,
	  6, 37, 39,
	 42, 47, 48,
	 42, 48, 43,
	 61, 46, 44,
	 45, 70, 69,
	 69, 70, 71,
	 69, 71, 49,
	 74, 78, 77,
	 83, 84, 85,
	 73, 74, 77,
	 93, 96, 92,
	 68, 46, 93,
	 95, 99, 63,
	 95, 63, 59,
	115,108,110,
	115,110,114,
	125,126,127,
	129,130,132,
	137,133,138,
	137,138,139,
	148,146,143,
	148,143,147,
	119,118,154,
	161,147,143,
	165,164,151,
	158,157,171,
	158,171,172,
	159,158,187,
	159,187,186,
	194,192,191,
	194,191,193,
	189,202,201,
	182,197,184,
	205,  8,  7,
	 48,109,107,
	218,219, 57,
	218, 57, 56,
	207,231,211,
	232,230,231,
	232,231,233,
	 53, 52, 31,
	388,411,386,
	409,430,250,
	262,429,254,
	262,254,256,
	442,444,428,
	273,264,383,
	273,383,384,
	429,271,251,
	429,251,254,
	413,365,362,
	 67,413,360,
	282,283,295,
	285,301,299,
	202,281,280,
	284,283,291,
	284,291,289,
	320,189,160,
	308,306,307,
	307,309,308,
	319,317,330,
	319,330,328,
	353,352,324,
	332,331,333,
	340,341,338,
	354,341,344,
	349,358,357,
	349,357,347,
	364,355,356,
	364,356,363,
	364,365,366,
	364,366,369,
	374,376,402,
	375, 92,373,
	 77,389,390,
	382,380,381,
	389, 77,386,
	393,394,412,
	393,412,392,
	401,394,416,
	415,400,403,
	411,410,427,
	411,427,426,
	422,420,424,
	247,248,263,
	247,263,261,
	445,443, 14,
	445, 14, 11,
	449,450,  4,
	449,  4,  5,
	443,441, 17,
	443, 17, 14,
	436, 23, 17,
	436, 17,441,
	424,448,422,
	448,423,422,
	414,419,418,
	414,418,413,
	406,404,405,
	399,397,395,
	399,395,396,
	420,416,392,
	388,410,411,
	386,384,383,
	390, 88, 77,
	375, 94, 92,
	415,414, 68,
	415, 68, 94,
	370,374,402,
	370,402,398,
	361,357,358,
	361,358,359,
	125,348,126,
	346,344,343,
	340,338,339,
	337,335,334,
	337,334,336,
	325,353,324,
	324,331,332,
	324,332,329,
	323,322,309,
	323,309,310,
	294,295,297,
	294,297,296,
	289,286,285,
	202,280,203,
	288,307,303,
	282,295,321,
	 67,360,111,
	418,423,367,
	418,367,366,
	272,252,251,
	272,251,271,
	272,271,270,
	255,253,274,
	265,266,380,
	265,380,382,
	442,428,261,
	440,263,258,
	440,258,260,
	409,250,410,
	255,226,245,
	255,245,246,
	 31,240,243,
	236,234,235,
	236,235,237,
	233,225,245,
	233,245,244,
	220,221, 85,
	220, 85, 86,
	 81,213,226,
	 81,226, 80,
	  7,206,205,
	186,184,198,
	186,198,199,
	204,203,205,
	204,205,206,
	195,193,196,
	171,174,172,
	173,174,175,
	173,172,174,
	155,167,166,
	160,161,143,
	160,143,144,
	119,154,155,
	148,151,150,
	148,150,146,
	140,137,139,
	140,139,141,
	127,126,130,
	114,124,128,
	114,128,115,
	117,105,106,
	117,106,116,
	104,105,100,
	104,100,103,
	 59, 60, 91,
	 97, 96, 61,
	 97, 61, 64,
	 91, 72, 89,
	 87, 84, 79,
	 87, 79, 76,
	 78, 80, 77,
	 49, 50, 74,
	 60, 44, 45,
	 61, 44, 58,
	 51, 50, 35,
	 51, 35, 34,
	 39, 37, 41,
	 33, 34,  9,
	 33,  9, 12,
	  0, 36, 37,
	  0, 37,  6,
	 40, 46, 47,
	 40, 47, 42,
	 53, 54, 56,
	 65, 62, 63,
	 72, 49, 73,
	 79, 78, 75,
	 79, 75, 76,
	 52, 53, 76,
	 92, 89, 90,
	 96, 93, 46,
	102,103,100,
	102,100,101,
	116,106,108,
	116,108,115,
	123,125,124,
	116,115,128,
	118,131,135,
	140,135,136,
	148,147,149,
	120,119,155,
	164,162,152,
	164,152,150,
	157,147,161,
	157,161,170,
	186,187,185,
	186,185,184,
	193,197,196,
	202,203,204,
	194,195,178,
	198,184,197,
	 67,111,109,
	 38, 43,103,
	 38,103,102,
	214,223,222,
	214,222,221,
	214,221,220,
	214,220,219,
	214,219,218,
	213,237,235,
	221,222, 83,
	221, 83, 85,
	 15,229, 33,
	227, 18,230,
	227,230,232,
	 52, 51,240,
	 75, 78, 50,
	408,430,409,
	260,258,257,
	260,257,259,
	224,207,259,
	268,269,405,
	268,405,404,
	413,362,360,
	447,  8,205,
	299,297,285,
	189,281,202,
	290,288,289,
	290,289,291,
	322,321,295,
	322,295,294,
	333,323,311,
	333,311,320,
	317,316,329,
	320,160,144,
	353,325,326,
	329,332,334,
	329,334,330,
	339,338,141,
	339,141,139,
	348,345,126,
	347,356,346,
	123,349,125,
	364,353,354,
	364,354,355,
	365,364,363,
	376,391,394,
	376,394,401,
	 92,376,374,
	 92,374,373,
	377, 90, 88,
	380,379,378,
	380,378,381,
	388,387,409,
	388,409,410,
	416,393,392,
	399,398,402,
	399,402,403,
	250,428,427,
	421,417,416,
	421,416,420,
	426,427,446,
	426,446,451,
	444,442,441,
	452,451,450,
	452,450,449
};


        //***************************THE END OF FAMOUS BUNNY TRIMESH********************************************//

        //*************************** NOT REALLY FAMOUS TORUS ********************************************//


        public static IndexedVector3[] gTorusVerticesArray = new IndexedVector3[TORUS_NUM_VERTICES] {
new IndexedVector3(2.5f,		0f,		0f),
new IndexedVector3(2.405f,	0.294f,	0f),
new IndexedVector3(2.155f,	0.476f,	0f),
new IndexedVector3(1.845f,	0.476f,	0f),
new IndexedVector3(1.595f,	0.294f,	0f),
new IndexedVector3(1.5f,		0f,		0f),
new IndexedVector3(1.595f,	-0.294f,	0f),
new IndexedVector3(1.845f,	-0.476f,	0f),
new IndexedVector3(2.155f,	-0.476f,	0f),
new IndexedVector3(2.405f,	-0.294f,	0f),
new IndexedVector3(2.445f,	0f,		0.52f),
new IndexedVector3(2.352f,	0.294f,	0.5f),
new IndexedVector3(2.107f,	0.476f,	0.448f),
new IndexedVector3(1.805f,	0.476f,	0.384f),
new IndexedVector3(1.561f,	0.294f,	0.332f),
new IndexedVector3(1.467f,	0f,		0.312f),
new IndexedVector3(1.561f,	-0.294f,	0.332f),
new IndexedVector3(1.805f,	-0.476f,	0.384f),
new IndexedVector3(2.107f,	-0.476f,	0.448f),
new IndexedVector3(2.352f,	-0.294f,	0.5f),
new IndexedVector3(2.284f,	0f,		1.017f),
new IndexedVector3(2.197f,	0.294f,	0.978f),
new IndexedVector3(1.968f,	0.476f,	0.876f),
new IndexedVector3(1.686f,	0.476f,	0.751f),
new IndexedVector3(1.458f,	0.294f,	0.649f),
new IndexedVector3(1.37f,		0f,		0.61f),
new IndexedVector3(1.458f,	-0.294f,	0.649f),
new IndexedVector3(1.686f,	-0.476f,	0.751f),
new IndexedVector3(1.968f,	-0.476f,	0.876f),
new IndexedVector3(2.197f,	-0.294f,	0.978f),
new IndexedVector3(2.023f,	0f,		1.469f),
new IndexedVector3(1.945f,	0.294f,	1.413f),
new IndexedVector3(1.743f,	0.476f,	1.266f),
new IndexedVector3(1.493f,	0.476f,	1.085f),
new IndexedVector3(1.291f,	0.294f,	0.938f),
new IndexedVector3(1.214f,	0f,		0.882f),
new IndexedVector3(1.291f,	-0.294f,	0.938f),
new IndexedVector3(1.493f,	-0.476f,	1.085f),
new IndexedVector3(1.743f,	-0.476f,	1.266f),
new IndexedVector3(1.945f,	-0.294f,	1.413f),
new IndexedVector3(1.673f,	0f,		1.858f),
new IndexedVector3(1.609f,	0.294f,	1.787f),
new IndexedVector3(1.442f,	0.476f,	1.601f),
new IndexedVector3(1.235f,	0.476f,	1.371f),
new IndexedVector3(1.068f,	0.294f,	1.186f),
new IndexedVector3(1.004f,	0f,		1.115f),
new IndexedVector3(1.068f,	-0.294f,	1.186f),
new IndexedVector3(1.235f,	-0.476f,	1.371f),
new IndexedVector3(1.442f,	-0.476f,	1.601f),
new IndexedVector3(1.609f,	-0.294f,	1.787f),
new IndexedVector3(1.25f,		0f,		2.165f),
new IndexedVector3(1.202f,	0.294f,	2.082f),
new IndexedVector3(1.077f,	0.476f,	1.866f),
new IndexedVector3(0.923f,	0.476f,	1.598f),
new IndexedVector3(0.798f,	0.294f,	1.382f),
new IndexedVector3(0.75f,		0f,		1.299f),
new IndexedVector3(0.798f,	-0.294f,	1.382f),
new IndexedVector3(0.923f,	-0.476f,	1.598f),
new IndexedVector3(1.077f,	-0.476f,	1.866f),
new IndexedVector3(1.202f,	-0.294f,	2.082f),
new IndexedVector3(0.773f,	0f,		2.378f),
new IndexedVector3(0.743f,	0.294f,	2.287f),
new IndexedVector3(0.666f,	0.476f,	2.049f),
new IndexedVector3(0.57f,		0.476f,	1.755f),
new IndexedVector3(0.493f,	0.294f,	1.517f),
new IndexedVector3(0.464f,	0f,		1.427f),
new IndexedVector3(0.493f,	-0.294f,	1.517f),
new IndexedVector3(0.57f,		-0.476f,	1.755f),
new IndexedVector3(0.666f,	-0.476f,	2.049f),
new IndexedVector3(0.743f,	-0.294f,	2.287f),
new IndexedVector3(0.261f,	0f,		2.486f),
new IndexedVector3(0.251f,	0.294f,	2.391f),
new IndexedVector3(0.225f,	0.476f,	2.143f),
new IndexedVector3(0.193f,	0.476f,	1.835f),
new IndexedVector3(0.167f,	0.294f,	1.587f),
new IndexedVector3(0.157f,	0f,		1.492f),
new IndexedVector3(0.167f,	-0.294f,	1.587f),
new IndexedVector3(0.193f,	-0.476f,	1.835f),
new IndexedVector3(0.225f,	-0.476f,	2.143f),
new IndexedVector3(0.251f,	-0.294f,	2.391f),
new IndexedVector3(-0.261f,	0f,		2.486f),
new IndexedVector3(-0.251f,	0.294f,	2.391f),
new IndexedVector3(-0.225f,	0.476f,	2.143f),
new IndexedVector3(-0.193f,	0.476f,	1.835f),
new IndexedVector3(-0.167f,	0.294f,	1.587f),
new IndexedVector3(-0.157f,	0f,		1.492f),
new IndexedVector3(-0.167f,	-0.294f,	1.587f),
new IndexedVector3(-0.193f,	-0.476f,	1.835f),
new IndexedVector3(-0.225f,	-0.476f,	2.143f),
new IndexedVector3(-0.251f,	-0.294f,	2.391f),
new IndexedVector3(-0.773f,	0f,		2.378f),
new IndexedVector3(-0.743f,	0.294f,	2.287f),
new IndexedVector3(-0.666f,	0.476f,	2.049f),
new IndexedVector3(-0.57f,	0.476f,	1.755f),
new IndexedVector3(-0.493f,	0.294f,	1.517f),
new IndexedVector3(-0.464f,	0f,		1.427f),
new IndexedVector3(-0.493f,	-0.294f,	1.517f),
new IndexedVector3(-0.57f,	-0.476f,	1.755f),
new IndexedVector3(-0.666f,	-0.476f,	2.049f),
new IndexedVector3(-0.743f,	-0.294f,	2.287f),
new IndexedVector3(-1.25f,	0f,		2.165f),
new IndexedVector3(-1.202f,	0.294f,	2.082f),
new IndexedVector3(-1.077f,	0.476f,	1.866f),
new IndexedVector3(-0.923f,	0.476f,	1.598f),
new IndexedVector3(-0.798f,	0.294f,	1.382f),
new IndexedVector3(-0.75f,	0f,		1.299f),
new IndexedVector3(-0.798f,	-0.294f,	1.382f),
new IndexedVector3(-0.923f,	-0.476f,	1.598f),
new IndexedVector3(-1.077f,	-0.476f,	1.866f),
new IndexedVector3(-1.202f,	-0.294f,	2.082f),
new IndexedVector3(-1.673f,	0f,		1.858f),
new IndexedVector3(-1.609f,	0.294f,	1.787f),
new IndexedVector3(-1.442f,	0.476f,	1.601f),
new IndexedVector3(-1.235f,	0.476f,	1.371f),
new IndexedVector3(-1.068f,	0.294f,	1.186f),
new IndexedVector3(-1.004f,	0f,		1.115f),
new IndexedVector3(-1.068f,	-0.294f,	1.186f),
new IndexedVector3(-1.235f,	-0.476f,	1.371f),
new IndexedVector3(-1.442f,	-0.476f,	1.601f),
new IndexedVector3(-1.609f,	-0.294f,	1.787f),
new IndexedVector3(-2.023f,	0f,		1.469f),
new IndexedVector3(-1.945f,	0.294f,	1.413f),
new IndexedVector3(-1.743f,	0.476f,	1.266f),
new IndexedVector3(-1.493f,	0.476f,	1.085f),
new IndexedVector3(-1.291f,	0.294f,	0.938f),
new IndexedVector3(-1.214f,	0f,		0.882f),
new IndexedVector3(-1.291f,	-0.294f,	0.938f),
new IndexedVector3(-1.493f,	-0.476f,	1.085f),
new IndexedVector3(-1.743f,	-0.476f,	1.266f),
new IndexedVector3(-1.945f,	-0.294f,	1.413f),
new IndexedVector3(-2.284f,	0f,		1.017f),
new IndexedVector3(-2.197f,	0.294f,	0.978f),
new IndexedVector3(-1.968f,	0.476f,	0.876f),
new IndexedVector3(-1.686f,	0.476f,	0.751f),
new IndexedVector3(-1.458f,	0.294f,	0.649f),
new IndexedVector3(-1.37f,	0f,		0.61f),
new IndexedVector3(-1.458f,	-0.294f,	0.649f),
new IndexedVector3(-1.686f,	-0.476f,	0.751f),
new IndexedVector3(-1.968f,	-0.476f,	0.876f),
new IndexedVector3(-2.197f,	-0.294f,	0.978f),
new IndexedVector3(-2.445f,	0f,		0.52f),
new IndexedVector3(-2.352f,	0.294f,	0.5f),
new IndexedVector3(-2.107f,	0.476f,	0.448f),
new IndexedVector3(-1.805f,	0.476f,	0.384f),
new IndexedVector3(-1.561f,	0.294f,	0.332f),
new IndexedVector3(-1.467f,	0f,		0.312f),
new IndexedVector3(-1.561f,	-0.294f,	0.332f),
new IndexedVector3(-1.805f,	-0.476f,	0.384f),
new IndexedVector3(-2.107f,	-0.476f,	0.448f),
new IndexedVector3(-2.352f,	-0.294f,	0.5f),
new IndexedVector3(-2.5f,	0f,		0f),
new IndexedVector3(-2.405f,	0.294f,	0f),
new IndexedVector3(-2.155f,	0.476f,	0f),
new IndexedVector3(-1.845f,	0.476f,	0f),
new IndexedVector3(-1.595f,	0.294f,	0f),
new IndexedVector3(-1.5f,		0f,		0f),
new IndexedVector3(-1.595f,	-0.294f,	0f),
new IndexedVector3(-1.845f,	-0.476f,	0f),
new IndexedVector3(-2.155f,	-0.476f,	0f),
new IndexedVector3(-2.405f,	-0.294f,	0f),
new IndexedVector3(-2.445f,	0f,		-0.52f),
new IndexedVector3(-2.352f,	0.294f,	-0.5f),
new IndexedVector3(-2.107f,	0.476f,	-0.448f),
new IndexedVector3(-1.805f,	0.476f,	-0.384f),
new IndexedVector3(-1.561f,	0.294f,	-0.332f),
new IndexedVector3(-1.467f,	0f,		-0.312f),
new IndexedVector3(-1.561f,	-0.294f,	-0.332f),
new IndexedVector3(-1.805f,	-0.476f,	-0.384f),
new IndexedVector3(-2.107f,	-0.476f,	-0.448f),
new IndexedVector3(-2.352f,	-0.294f,	-0.5f),
new IndexedVector3(-2.284f,	0f,		-1.017f),
new IndexedVector3(-2.197f,	0.294f,	-0.978f),
new IndexedVector3(-1.968f,	0.476f,	-0.876f),
new IndexedVector3(-1.686f,	0.476f,	-0.751f),
new IndexedVector3(-1.458f,	0.294f,	-0.649f),
new IndexedVector3(-1.37f,	0f,		-0.61f),
new IndexedVector3(-1.458f,	-0.294f,	-0.649f),
new IndexedVector3(-1.686f,	-0.476f,	-0.751f),
new IndexedVector3(-1.968f,	-0.476f,	-0.876f),
new IndexedVector3(-2.197f,	-0.294f,	-0.978f),
new IndexedVector3(-2.023f,	0f,		-1.469f),
new IndexedVector3(-1.945f,	0.294f,	-1.413f),
new IndexedVector3(-1.743f,	0.476f,	-1.266f),
new IndexedVector3(-1.493f,	0.476f,	-1.085f),
new IndexedVector3(-1.291f,	0.294f,	-0.938f),
new IndexedVector3(-1.214f,	0f,		-0.882f),
new IndexedVector3(-1.291f,	-0.294f,	-0.938f),
new IndexedVector3(-1.493f,	-0.476f,	-1.085f),
new IndexedVector3(-1.743f,	-0.476f,	-1.266f),
new IndexedVector3(-1.945f,	-0.294f,	-1.413f),
new IndexedVector3(-1.673f,	0f,		-1.858f),
new IndexedVector3(-1.609f,	0.294f,	-1.787f),
new IndexedVector3(-1.442f,	0.476f,	-1.601f),
new IndexedVector3(-1.235f,	0.476f,	-1.371f),
new IndexedVector3(-1.068f,	0.294f,	-1.186f),
new IndexedVector3(-1.004f,	0f,		-1.115f),
new IndexedVector3(-1.068f,	-0.294f,	-1.186f),
new IndexedVector3(-1.235f,	-0.476f,	-1.371f),
new IndexedVector3(-1.442f,	-0.476f,	-1.601f),
new IndexedVector3(-1.609f,	-0.294f,	-1.787f),
new IndexedVector3(-1.25f,	0f,		-2.165f),
new IndexedVector3(-1.202f,	0.294f,	-2.082f),
new IndexedVector3(-1.077f,	0.476f,	-1.866f),
new IndexedVector3(-0.923f,	0.476f,	-1.598f),
new IndexedVector3(-0.798f,	0.294f,	-1.382f),
new IndexedVector3(-0.75f,	0f,		-1.299f),
new IndexedVector3(-0.798f,	-0.294f,	-1.382f),
new IndexedVector3(-0.923f,	-0.476f,	-1.598f),
new IndexedVector3(-1.077f,	-0.476f,	-1.866f),
new IndexedVector3(-1.202f,	-0.294f,	-2.082f),
new IndexedVector3(-0.773f,	0f,		-2.378f),
new IndexedVector3(-0.743f,	0.294f,	-2.287f),
new IndexedVector3(-0.666f,	0.476f,	-2.049f),
new IndexedVector3(-0.57f,	0.476f,	-1.755f),
new IndexedVector3(-0.493f,	0.294f,	-1.517f),
new IndexedVector3(-0.464f,	0f,		-1.427f),
new IndexedVector3(-0.493f,	-0.294f,	-1.517f),
new IndexedVector3(-0.57f,	-0.476f,	-1.755f),
new IndexedVector3(-0.666f,	-0.476f,	-2.049f),
new IndexedVector3(-0.743f,	-0.294f,	-2.287f),
new IndexedVector3(-0.261f,	0f,		-2.486f),
new IndexedVector3(-0.251f,	0.294f,	-2.391f),
new IndexedVector3(-0.225f,	0.476f,	-2.143f),
new IndexedVector3(-0.193f,	0.476f,	-1.835f),
new IndexedVector3(-0.167f,	0.294f,	-1.587f),
new IndexedVector3(-0.157f,	0f,		-1.492f),
new IndexedVector3(-0.167f,	-0.294f,	-1.587f),
new IndexedVector3(-0.193f,	-0.476f,	-1.835f),
new IndexedVector3(-0.225f,	-0.476f,	-2.143f),
new IndexedVector3(-0.251f,	-0.294f,	-2.391f),
new IndexedVector3(0.261f,	0f,		-2.486f),
new IndexedVector3(0.251f,	0.294f,	-2.391f),
new IndexedVector3(0.225f,	0.476f,	-2.143f),
new IndexedVector3(0.193f,	0.476f,	-1.835f),
new IndexedVector3(0.167f,	0.294f,	-1.587f),
new IndexedVector3(0.157f,	0f,		-1.492f),
new IndexedVector3(0.167f,	-0.294f,	-1.587f),
new IndexedVector3(0.193f,	-0.476f,	-1.835f),
new IndexedVector3(0.225f,	-0.476f,	-2.143f),
new IndexedVector3(0.251f,	-0.294f,	-2.391f),
new IndexedVector3(0.773f,	0f,		-2.378f),
new IndexedVector3(0.743f,	0.294f,	-2.287f),
new IndexedVector3(0.666f,	0.476f,	-2.049f),
new IndexedVector3(0.57f,		0.476f,	-1.755f),
new IndexedVector3(0.493f,	0.294f,	-1.517f),
new IndexedVector3(0.464f,	0f,		-1.427f),
new IndexedVector3(0.493f,	-0.294f,	-1.517f),
new IndexedVector3(0.57f,		-0.476f,	-1.755f),
new IndexedVector3(0.666f,	-0.476f,	-2.049f),
new IndexedVector3(0.743f,	-0.294f,	-2.287f),
new IndexedVector3(1.25f,		0f,		-2.165f),
new IndexedVector3(1.202f,	0.294f,	-2.082f),
new IndexedVector3(1.077f,	0.476f,	-1.866f),
new IndexedVector3(0.923f,	0.476f,	-1.598f),
new IndexedVector3(0.798f,	0.294f,	-1.382f),
new IndexedVector3(0.75f,		0f,		-1.299f),
new IndexedVector3(0.798f,	-0.294f,	-1.382f),
new IndexedVector3(0.923f,	-0.476f,	-1.598f),
new IndexedVector3(1.077f,	-0.476f,	-1.866f),
new IndexedVector3(1.202f,	-0.294f,	-2.082f),
new IndexedVector3(1.673f,	0f,		-1.858f),
new IndexedVector3(1.609f,	0.294f,	-1.787f),
new IndexedVector3(1.442f,	0.476f,	-1.601f),
new IndexedVector3(1.235f,	0.476f,	-1.371f),
new IndexedVector3(1.068f,	0.294f,	-1.186f),
new IndexedVector3(1.004f,	0f,		-1.115f),
new IndexedVector3(1.068f,	-0.294f,	-1.186f),
new IndexedVector3(1.235f,	-0.476f,	-1.371f),
new IndexedVector3(1.442f,	-0.476f,	-1.601f),
new IndexedVector3(1.609f,	-0.294f,	-1.787f),
new IndexedVector3(2.023f,	0f,		-1.469f),
new IndexedVector3(1.945f,	0.294f,	-1.413f),
new IndexedVector3(1.743f,	0.476f,	-1.266f),
new IndexedVector3(1.493f,	0.476f,	-1.085f),
new IndexedVector3(1.291f,	0.294f,	-0.938f),
new IndexedVector3(1.214f,	0f,		-0.882f),
new IndexedVector3(1.291f,	-0.294f,	-0.938f),
new IndexedVector3(1.493f,	-0.476f,	-1.085f),
new IndexedVector3(1.743f,	-0.476f,	-1.266f),
new IndexedVector3(1.945f,	-0.294f,	-1.413f),
new IndexedVector3(2.284f,	0f,		-1.017f),
new IndexedVector3(2.197f,	0.294f,	-0.978f),
new IndexedVector3(1.968f,	0.476f,	-0.876f),
new IndexedVector3(1.686f,	0.476f,	-0.751f),
new IndexedVector3(1.458f,	0.294f,	-0.649f),
new IndexedVector3(1.37f,		0f,		-0.61f),
new IndexedVector3(1.458f,	-0.294f,	-0.649f),
new IndexedVector3(1.686f,	-0.476f,	-0.751f),
new IndexedVector3(1.968f,	-0.476f,	-0.876f),
new IndexedVector3(2.197f,	-0.294f,	-0.978f),
new IndexedVector3(2.445f,	0f,		-0.52f),
new IndexedVector3(2.352f,	0.294f,	-0.5f),
new IndexedVector3(2.107f,	0.476f,	-0.448f),
new IndexedVector3(1.805f,	0.476f,	-0.384f),
new IndexedVector3(1.561f,	0.294f,	-0.332f),
new IndexedVector3(1.467f,	0f,		-0.312f),
new IndexedVector3(1.561f,	-0.294f,	-0.332f),
new IndexedVector3(1.805f,	-0.476f,	-0.384f),
new IndexedVector3(2.107f,	-0.476f,	-0.448f),
new IndexedVector3(2.352f,	-0.294f,	-0.5f)
};


        public static int[] gTorusIndicesArray = new int[TORUS_NUM_TRIANGLES * 3] {
    0,    1,    11,
    1,    2,    12,
	2,    3,    13,
	3,    4,    14,
	4,    5,    15,
	5,    6,    16,
	6,    7,    17,
	7,    8,    18,
	8,    9,    19,
	9,    0,    10,
	10,    11,    21,
	11,    12,    22,
	12,    13,    23,
	13,    14,    24,
	14,    15,    25,
	15,    16,    26,
	16,    17,    27,
	17,    18,    28,
	18,    19,    29,
	19,    10,    20,
	20,    21,    31,
	21,    22,    32,
	22,    23,    33,
	23,    24,    34,
	24,    25,    35,
	25,    26,    36,
	26,    27,    37,
	27,    28,    38,
	28,    29,    39,
	29,    20,    30,
	30,    31,    41,
	31,    32,    42,
	32,    33,    43,
	33,    34,    44,
	34,    35,    45,
	35,    36,    46,
	36,    37,    47,
	37,    38,    48,
	38,    39,    49,
	39,    30,    40,
	40,    41,    51,
	41,    42,    52,
	42,    43,    53,
	43,    44,    54,
	44,    45,    55,
	45,    46,    56,
	46,    47,    57,
	47,    48,    58,
	48,    49,    59,
	49,    40,    50,
	50,    51,    61,
	51,    52,    62,
	52,    53,    63,
	53,    54,    64,
	54,    55,    65,
	55,    56,    66,
	56,    57,    67,
	57,    58,    68,
	58,    59,    69,
	59,    50,    60,
	60,    61,    71,
	61,    62,    72,
	62,    63,    73,
	63,    64,    74,
	64,    65,    75,
	65,    66,    76,
	66,    67,    77,
	67,    68,    78,
	68,    69,    79,
	69,    60,    70,
	70,    71,    81,
	71,    72,    82,
	72,    73,    83,
	73,    74,    84,
	74,    75,    85,
	75,    76,    86,
	76,    77,    87,
	77,    78,    88,
	78,    79,    89,
	79,    70,    80,
	80,    81,    91,
	81,    82,    92,
	82,    83,    93,
	83,    84,    94,
	84,    85,    95,
	85,    86,    96,
	86,    87,    97,
	87,    88,    98,
	88,    89,    99,
	89,    80,    90,
	90,    91,    101,
	91,    92,    102,
	92,    93,    103,
	93,    94,    104,
	94,    95,    105,
	95,    96,    106,
	96,    97,    107,
	97,    98,    108,
	98,    99,    109,
	99,    90,    100,
	100,    101,    111,
	101,    102,    112,
	102,    103,    113,
	103,    104,    114,
	104,    105,    115,
	105,    106,    116,
	106,    107,    117,
	107,    108,    118,
	108,    109,    119,
	109,    100,    110,
	110,    111,    121,
	111,    112,    122,
	112,    113,    123,
	113,    114,    124,
	114,    115,    125,
	115,    116,    126,
	116,    117,    127,
	117,    118,    128,
	118,    119,    129,
	119,    110,    120,
	120,    121,    131,
	121,    122,    132,
	122,    123,    133,
	123,    124,    134,
	124,    125,    135,
	125,    126,    136,
	126,    127,    137,
	127,    128,    138,
	128,    129,    139,
	129,    120,    130,
	130,    131,    141,
	131,    132,    142,
	132,    133,    143,
	133,    134,    144,
	134,    135,    145,
	135,    136,    146,
	136,    137,    147,
	137,    138,    148,
	138,    139,    149,
	139,    130,    140,
	140,    141,    151,
	141,    142,    152,
	142,    143,    153,
	143,    144,    154,
	144,    145,    155,
	145,    146,    156,
	146,    147,    157,
	147,    148,    158,
	148,    149,    159,
	149,    140,    150,
	150,    151,    161,
	151,    152,    162,
	152,    153,    163,
	153,    154,    164,
	154,    155,    165,
	155,    156,    166,
	156,    157,    167,
	157,    158,    168,
	158,    159,    169,
	159,    150,    160,
	160,    161,    171,
	161,    162,    172,
	162,    163,    173,
	163,    164,    174,
	164,    165,    175,
	165,    166,    176,
	166,    167,    177,
	167,    168,    178,
	168,    169,    179,
	169,    160,    170,
	170,    171,    181,
	171,    172,    182,
	172,    173,    183,
	173,    174,    184,
	174,    175,    185,
	175,    176,    186,
	176,    177,    187,
	177,    178,    188,
	178,    179,    189,
	179,    170,    180,
	180,    181,    191,
	181,    182,    192,
	182,    183,    193,
	183,    184,    194,
	184,    185,    195,
	185,    186,    196,
	186,    187,    197,
	187,    188,    198,
	188,    189,    199,
	189,    180,    190,
	190,    191,    201,
	191,    192,    202,
	192,    193,    203,
	193,    194,    204,
	194,    195,    205,
	195,    196,    206,
	196,    197,    207,
	197,    198,    208,
	198,    199,    209,
	199,    190,    200,
	200,    201,    211,
	201,    202,    212,
	202,    203,    213,
	203,    204,    214,
	204,    205,    215,
	205,    206,    216,
	206,    207,    217,
	207,    208,    218,
	208,    209,    219,
	209,    200,    210,
	210,    211,    221,
	211,    212,    222,
	212,    213,    223,
	213,    214,    224,
	214,    215,    225,
	215,    216,    226,
	216,    217,    227,
	217,    218,    228,
	218,    219,    229,
	219,    210,    220,
	220,    221,    231,
	221,    222,    232,
	222,    223,    233,
	223,    224,    234,
	224,    225,    235,
	225,    226,    236,
	226,    227,    237,
	227,    228,    238,
	228,    229,    239,
	229,    220,    230,
	230,    231,    241,
	231,    232,    242,
	232,    233,    243,
	233,    234,    244,
	234,    235,    245,
	235,    236,    246,
	236,    237,    247,
	237,    238,    248,
	238,    239,    249,
	239,    230,    240,
	240,    241,    251,
	241,    242,    252,
	242,    243,    253,
	243,    244,    254,
	244,    245,    255,
	245,    246,    256,
	246,    247,    257,
	247,    248,    258,
	248,    249,    259,
	249,    240,    250,
	250,    251,    261,
	251,    252,    262,
	252,    253,    263,
	253,    254,    264,
	254,    255,    265,
	255,    256,    266,
	256,    257,    267,
	257,    258,    268,
	258,    259,    269,
	259,    250,    260,
	260,    261,    271,
	261,    262,    272,
	262,    263,    273,
	263,    264,    274,
	264,    265,    275,
	265,    266,    276,
	266,    267,    277,
	267,    268,    278,
	268,    269,    279,
	269,    260,    270,
	270,    271,    281,
	271,    272,    282,
	272,    273,    283,
	273,    274,    284,
	274,    275,    285,
	275,    276,    286,
	276,    277,    287,
	277,    278,    288,
	278,    279,    289,
	279,    270,    280,
	280,    281,    291,
	281,    282,    292,
	282,    283,    293,
	283,    284,    294,
	284,    285,    295,
	285,    286,    296,
	286,    287,    297,
	287,    288,    298,
	288,    289,    299,
	289,    280,    290,
	290,    291,    1,
	291,    292,    2,
	292,    293,    3,
	293,    294,    4,
	294,    295,    5,
	295,    296,    6,
	296,    297,    7,
	297,    298,    8,
	298,    299,    9,
	299,    290,    0,
	0,    11,    10,
	1,    12,    11,
	2,    13,    12,
	3,    14,    13,
	4,    15,    14,
	5,    16,    15,
	6,    17,    16,
	7,    18,    17,
	8,    19,    18,
	9,    10,    19,
	10,    21,    20,
	11,    22,    21,
	12,    23,    22,
	13,    24,    23,
	14,    25,    24,
	15,    26,    25,
	16,    27,    26,
	17,    28,    27,
	18,    29,    28,
	19,    20,    29,
	20,    31,    30,
	21,    32,    31,
	22,    33,    32,
	23,    34,    33,
	24,    35,    34,
	25,    36,    35,
	26,    37,    36,
	27,    38,    37,
	28,    39,    38,
	29,    30,    39,
	30,    41,    40,
	31,    42,    41,
	32,    43,    42,
	33,    44,    43,
	34,    45,    44,
	35,    46,    45,
	36,    47,    46,
	37,    48,    47,
	38,    49,    48,
	39,    40,    49,
	40,    51,    50,
	41,    52,    51,
	42,    53,    52,
	43,    54,    53,
	44,    55,    54,
	45,    56,    55,
	46,    57,    56,
	47,    58,    57,
	48,    59,    58,
	49,    50,    59,
	50,    61,    60,
	51,    62,    61,
	52,    63,    62,
	53,    64,    63,
	54,    65,    64,
	55,    66,    65,
	56,    67,    66,
	57,    68,    67,
	58,    69,    68,
	59,    60,    69,
	60,    71,    70,
	61,    72,    71,
	62,    73,    72,
	63,    74,    73,
	64,    75,    74,
	65,    76,    75,
	66,    77,    76,
	67,    78,    77,
	68,    79,    78,
	69,    70,    79,
	70,    81,    80,
	71,    82,    81,
	72,    83,    82,
	73,    84,    83,
	74,    85,    84,
	75,    86,    85,
	76,    87,    86,
	77,    88,    87,
	78,    89,    88,
	79,    80,    89,
	80,    91,    90,
	81,    92,    91,
	82,    93,    92,
	83,    94,    93,
	84,    95,    94,
	85,    96,    95,
	86,    97,    96,
	87,    98,    97,
	88,    99,    98,
	89,    90,    99,
	90,    101,    100,
	91,    102,    101,
	92,    103,    102,
	93,    104,    103,
	94,    105,    104,
	95,    106,    105,
	96,    107,    106,
	97,    108,    107,
	98,    109,    108,
	99,    100,    109,
	100,    111,    110,
	101,    112,    111,
	102,    113,    112,
	103,    114,    113,
	104,    115,    114,
	105,    116,    115,
	106,    117,    116,
	107,    118,    117,
	108,    119,    118,
	109,    110,    119,
	110,    121,    120,
	111,    122,    121,
	112,    123,    122,
	113,    124,    123,
	114,    125,    124,
	115,    126,    125,
	116,    127,    126,
	117,    128,    127,
	118,    129,    128,
	119,    120,    129,
	120,    131,    130,
	121,    132,    131,
	122,    133,    132,
	123,    134,    133,
	124,    135,    134,
	125,    136,    135,
	126,    137,    136,
	127,    138,    137,
	128,    139,    138,
	129,    130,    139,
	130,    141,    140,
	131,    142,    141,
	132,    143,    142,
	133,    144,    143,
	134,    145,    144,
	135,    146,    145,
	136,    147,    146,
	137,    148,    147,
	138,    149,    148,
	139,    140,    149,
	140,    151,    150,
	141,    152,    151,
	142,    153,    152,
	143,    154,    153,
	144,    155,    154,
	145,    156,    155,
	146,    157,    156,
	147,    158,    157,
	148,    159,    158,
	149,    150,    159,
	150,    161,    160,
	151,    162,    161,
	152,    163,    162,
	153,    164,    163,
	154,    165,    164,
	155,    166,    165,
	156,    167,    166,
	157,    168,    167,
	158,    169,    168,
	159,    160,    169,
	160,    171,    170,
	161,    172,    171,
	162,    173,    172,
	163,    174,    173,
	164,    175,    174,
	165,    176,    175,
	166,    177,    176,
	167,    178,    177,
	168,    179,    178,
	169,    170,    179,
	170,    181,    180,
	171,    182,    181,
	172,    183,    182,
	173,    184,    183,
	174,    185,    184,
	175,    186,    185,
	176,    187,    186,
	177,    188,    187,
	178,    189,    188,
	179,    180,    189,
	180,    191,    190,
	181,    192,    191,
	182,    193,    192,
	183,    194,    193,
	184,    195,    194,
	185,    196,    195,
	186,    197,    196,
	187,    198,    197,
	188,    199,    198,
	189,    190,    199,
	190,    201,    200,
	191,    202,    201,
	192,    203,    202,
	193,    204,    203,
	194,    205,    204,
	195,    206,    205,
	196,    207,    206,
	197,    208,    207,
	198,    209,    208,
	199,    200,    209,
	200,    211,    210,
	201,    212,    211,
	202,    213,    212,
	203,    214,    213,
	204,    215,    214,
	205,    216,    215,
	206,    217,    216,
	207,    218,    217,
	208,    219,    218,
	209,    210,    219,
	210,    221,    220,
	211,    222,    221,
	212,    223,    222,
	213,    224,    223,
	214,    225,    224,
	215,    226,    225,
	216,    227,    226,
	217,    228,    227,
	218,    229,    228,
	219,    220,    229,
	220,    231,    230,
	221,    232,    231,
	222,    233,    232,
	223,    234,    233,
	224,    235,    234,
	225,    236,    235,
	226,    237,    236,
	227,    238,    237,
	228,    239,    238,
	229,    230,    239,
	230,    241,    240,
	231,    242,    241,
	232,    243,    242,
	233,    244,    243,
	234,    245,    244,
	235,    246,    245,
	236,    247,    246,
	237,    248,    247,
	238,    249,    248,
	239,    240,    249,
	240,    251,    250,
	241,    252,    251,
	242,    253,    252,
	243,    254,    253,
	244,    255,    254,
	245,    256,    255,
	246,    257,    256,
	247,    258,    257,
	248,    259,    258,
	249,    250,    259,
	250,    261,    260,
	251,    262,    261,
	252,    263,    262,
	253,    264,    263,
	254,    265,    264,
	255,    266,    265,
	256,    267,    266,
	257,    268,    267,
	258,    269,    268,
	259,    260,    269,
	260,    271,    270,
	261,    272,    271,
	262,    273,    272,
	263,    274,    273,
	264,    275,    274,
	265,    276,    275,
	266,    277,    276,
	267,    278,    277,
	268,    279,    278,
	269,    270,    279,
	270,    281,    280,
	271,    282,    281,
	272,    283,    282,
	273,    284,    283,
	274,    285,    284,
	275,    286,    285,
	276,    287,    286,
	277,    288,    287,
	278,    289,    288,
	279,    280,    289,
	280,    291,    290,
	281,    292,    291,
	282,    293,    292,
	283,    294,    293,
	284,    295,    294,
	285,    296,    295,
	286,    297,    296,
	287,    298,    297,
	288,    299,    298,
	289,    290,    299,
	290,    1,	   0,
	291,    2,    1,
	292,    3,    2,
	293,    4,    3,
	294,    5,    4,
	295,    6,    5,
	296,    7,    6,
	297,    8,    7,
	298,    9,    8,
	299,    0,    9,
};



public static IndexedVector3[] TaruVtx = new IndexedVector3[]{
new IndexedVector3(1.08664f,-1.99237f,0.0f),
new IndexedVector3(0.768369f,-1.99237f,-0.768369f),
new IndexedVector3(1.28852f,1.34412e-007f,-1.28852f),
new IndexedVector3(1.82224f,1.90735e-007f,0.0f),
new IndexedVector3(0.0f,-1.99237f,-1.08664f),
new IndexedVector3(0.0f,0.0f,-1.82224f),
new IndexedVector3(0.0f,-1.99237f,-1.08664f),
new IndexedVector3(-0.768369f,-1.99237f,-0.768369f),
new IndexedVector3(-1.28852f,1.34412e-007f,-1.28852f),
new IndexedVector3(0.0f,0.0f,-1.82224f),
new IndexedVector3(-1.08664f,-1.99237f,1.82086e-007f),
new IndexedVector3(-1.82224f,1.90735e-007f,1.59305e-007f),
new IndexedVector3(-0.768369f,-1.99237f,0.76837f),
new IndexedVector3(-1.28852f,2.47058e-007f,1.28852f),
new IndexedVector3(1.42495e-007f,-1.99237f,1.08664f),
new IndexedVector3(2.38958e-007f,2.70388e-007f,1.82224f),
new IndexedVector3(0.768369f,-1.99237f,0.768369f),
new IndexedVector3(1.28852f,2.47058e-007f,1.28852f),
new IndexedVector3(0.768369f,1.99237f,-0.768369f),
new IndexedVector3(1.08664f,1.99237f,0.0f),
new IndexedVector3(0.0f,1.99237f,-1.08664f),
new IndexedVector3(-0.768369f,1.99237f,-0.768369f),
new IndexedVector3(0.0f,1.99237f,-1.08664f),
new IndexedVector3(-1.08664f,1.99237f,0.0f),
new IndexedVector3(-0.768369f,1.99237f,0.768369f),
new IndexedVector3(1.42495e-007f,1.99237f,1.08664f),
new IndexedVector3(0.768369f,1.99237f,0.768369f),
new IndexedVector3(1.42495e-007f,-1.99237f,1.08664f),
new IndexedVector3(-0.768369f,-1.99237f,0.76837f),
new IndexedVector3(-1.08664f,-1.99237f,1.82086e-007f),
new IndexedVector3(-0.768369f,-1.99237f,-0.768369f),
new IndexedVector3(0.0f,-1.99237f,-1.08664f),
new IndexedVector3(0.768369f,-1.99237f,-0.768369f),
new IndexedVector3(1.08664f,-1.99237f,0.0f),
new IndexedVector3(0.768369f,-1.99237f,0.768369f),
new IndexedVector3(0.768369f,1.99237f,-0.768369f),
new IndexedVector3(0.0f,1.99237f,-1.08664f),
new IndexedVector3(-0.768369f,1.99237f,-0.768369f),
new IndexedVector3(-1.08664f,1.99237f,0.0f),
new IndexedVector3(-0.768369f,1.99237f,0.768369f),
new IndexedVector3(1.42495e-007f,1.99237f,1.08664f),
new IndexedVector3(0.768369f,1.99237f,0.768369f),
new IndexedVector3(1.08664f,1.99237f,0.0f)
};




    }
}
